@@ -8,12 +8,12 @@
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the:
  *
@@ -48,25 +48,17 @@
 
 static int is_file(const char *filename);
 
-static void add_files(GenerateOptions *gop, XConfigPtr config);
 static void add_font_path(GenerateOptions *gop, XConfigPtr config);
 static void add_modules(GenerateOptions *gop, XConfigPtr config);
 
-static XConfigMonitorPtr
-add_monitor(XConfigPtr config, int count);
-
 static XConfigDevicePtr
 add_device(XConfigPtr config, int bus, int slot, char *boardname, int count);
 
-static XConfigDisplayPtr add_display(int depth);
 static void add_layout(GenerateOptions *gop, XConfigPtr config);
 
 static void add_inputref(XConfigPtr config, XConfigLayoutPtr layout,
                          char *name, char *coreKeyword);
 
-static int add_keyboard(GenerateOptions *gop, XConfigPtr config);
-static int add_mouse(GenerateOptions *gop, XConfigPtr config);
-
 /*
  * xconfigGenerate() - generate a new XConfig from scratch
  */
@@ -74,19 +66,19 @@ static int add_mouse(GenerateOptions *gop, XConfigPtr config);
 XConfigPtr xconfigGenerate(GenerateOptions *gop)
 {
     XConfigPtr config;
-    
+
     config = xconfigAlloc(sizeof(XConfigRec));
 
     /* add files, fonts, and modules */
 
-    add_files(gop, config);
+    config->files = xconfigAlloc(sizeof(XConfigFilesRec));
     add_font_path(gop, config);
     add_modules(gop, config);
 
     /* add the keyboard and mouse */
 
-    add_keyboard(gop, config);
-    add_mouse(gop, config);
+    xconfigAddKeyboard(gop, config);
+    xconfigAddMouse(gop, config);
 
     /* add the layout */
 
@@ -113,34 +105,34 @@ XConfigScreenPtr xconfigGenerateAddScreen(XConfigPtr config,
     XConfigScreenPtr screen, s;
     XConfigDevicePtr device;
     XConfigMonitorPtr monitor;
-    
-    monitor = add_monitor(config, count);
+
+    monitor = xconfigAddMonitor(config, count);
     device = add_device(config, bus, slot, boardname, count);
-    
+
     screen = xconfigAlloc(sizeof(XConfigScreenRec));
 
     screen->identifier = xconfigAlloc(32);
     snprintf(screen->identifier, 32, SCREEN_IDENTIFIER, count);
-    
+
     screen->device_name = xconfigStrdup(device->identifier);
     screen->device = device;
 
     screen->monitor_name = xconfigStrdup(monitor->identifier);
     screen->monitor = monitor;
-        
+
     screen->defaultdepth = 24;
-    
-    screen->displays = add_display(screen->defaultdepth);
+
+    xconfigAddDisplay(&screen->displays, screen->defaultdepth);
 
     /* append to the end of the screen list */
-    
+
     if (!config->screens) {
         config->screens = screen;
     } else {
         for (s = config->screens; s->next; s = s->next);
         s->next = screen;
     }
-    
+
     return screen;
 
 } /* xconfigGenerateAddScreen() */
@@ -148,24 +140,25 @@ XConfigScreenPtr xconfigGenerateAddScreen(XConfigPtr config,
 
 
 /*
- * assign_screen_adjacencies() - setup all the adjacency information
- * for the X screens in the given layout.  Nothing fancy here: just
- * position all the screens horizontally, moving from left to right.
+ * xconfigGenerateAssignScreenAdjacencies() - setup all the adjacency
+ * information for the X screens in the given layout.  Nothing fancy
+ * here: just position all the screens horizontally, moving from left
+ * to right.
  */
 
 void xconfigGenerateAssignScreenAdjacencies(XConfigLayoutPtr layout)
 {
     XConfigAdjacencyPtr adj, prev = NULL;
-    
+
     for (adj = layout->adjacencies; adj; adj = adj->next) {
-        
+
         if (prev) {
             adj->where = CONF_ADJ_RIGHTOF;
             adj->refscreen = xconfigStrdup(prev->screen_name);
         } else {
             adj->x = adj->y = -1;
         }
-        
+
         /* make sure all the obsolete positioning is empty */
 
         adj->top = NULL;
@@ -176,10 +169,10 @@ void xconfigGenerateAssignScreenAdjacencies(XConfigLayoutPtr layout)
         adj->left_name = NULL;
         adj->right = NULL;
         adj->right_name = NULL;
-        
+
         prev = adj;
     }
-    
+
 } /* xconfigGenerateAssignScreenAdjacencies() */
 
 
@@ -195,21 +188,69 @@ void xconfigGenerateAssignScreenAdjacencies(XConfigLayoutPtr layout)
 static int is_file(const char *filename)
 {
     return (access(filename, F_OK) == 0);
-    
+
 } /* is_file() */
 
 
-
 /*
- * add_files() - 
+ * find_libdir() - attempt to find the X server library path; this is
+ * either
+ *
+ *     `pkg-config --variable=libdir xorg-server`
+ *
+ * or
+ *
+ *     [X PROJECT ROOT]/lib
  */
 
-static void add_files(GenerateOptions *gop, XConfigPtr config)
+static char *find_libdir(GenerateOptions *gop)
 {
-    config->files = xconfigAlloc(sizeof(XConfigFilesRec));
-    config->files->rgbpath = xconfigStrcat(gop->x_project_root,
-                                      "/lib/X11/rgb", NULL);
-} /* add_files() */
+    struct stat stat_buf;
+    FILE *stream = NULL;
+    char *s, *libdir = NULL;
+
+    /*
+     * run the pkg-config command and read the output; if the output
+     * is a directory, then return that as the libdir
+     */
+
+    stream = popen("pkg-config --variable=libdir xorg-server", "r");
+
+    if (stream) {
+        char buf[256];
+
+        buf[0] = '\0';
+
+        while (1) {
+            if (fgets(buf, 255, stream) == NULL) break;
+
+            if (buf[0] != '\0') {
+
+                /* truncate any newline */
+
+                s = strchr(buf, '\n');
+                if (s) *s = '\0';
+
+                if ((stat(buf, &stat_buf) == 0) &&
+                    (S_ISDIR(stat_buf.st_mode))) {
+
+                    libdir = xconfigStrdup(buf);
+                    break;
+                }
+            }
+        }
+
+        pclose(stream);
+
+        if (libdir) return libdir;
+    }
+
+    /* otherwise, just fallback to [X PROJECT ROOT]/lib */
+
+    return xconfigStrcat(gop->x_project_root, "/lib", NULL);
+
+} /* find_libdir() */
+
 
 
 /*
@@ -222,33 +263,36 @@ static void add_files(GenerateOptions *gop, XConfigPtr config)
 static void add_font_path(GenerateOptions *gop, XConfigPtr config)
 {
     int i, ret;
-    char *path, *p, *orig, *fonts_dir;
-    
+    char *path, *p, *orig, *fonts_dir, *libdir;
+
     /*
      * The below font path has been constructed from various examples
      * and uses some suggests from the Font De-uglification HOWTO
      */
-    
+
     static const char *__font_paths[] = {
-        "ROOT/lib/X11/fonts/local/",
-        "ROOT/lib/X11/fonts/misc/:unscaled",
-        "ROOT/lib/X11/fonts/100dpi/:unscaled",
-        "ROOT/lib/X11/fonts/75dpi/:unscaled",
-        "ROOT/lib/X11/fonts/misc/",
-        "ROOT/lib/X11/fonts/Type1/",
-        "ROOT/lib/X11/fonts/CID/",
-        "ROOT/lib/X11/fonts/Speedo/",
-        "ROOT/lib/X11/fonts/100dpi/",
-        "ROOT/lib/X11/fonts/75dpi/",
-        "ROOT/lib/X11/fonts/cyrillic/",
-        "ROOT/lib/X11/fonts/TTF/",
-        "ROOT/lib/X11/fonts/truetype/",
+        "LIBDIR/X11/fonts/local/",
+        "LIBDIR/X11/fonts/misc/:unscaled",
+        "LIBDIR/X11/fonts/100dpi/:unscaled",
+        "LIBDIR/X11/fonts/75dpi/:unscaled",
+        "LIBDIR/X11/fonts/misc/",
+        "LIBDIR/X11/fonts/Type1/",
+        "LIBDIR/X11/fonts/CID/",
+        "LIBDIR/X11/fonts/Speedo/",
+        "LIBDIR/X11/fonts/100dpi/",
+        "LIBDIR/X11/fonts/75dpi/",
+        "LIBDIR/X11/fonts/cyrillic/",
+        "LIBDIR/X11/fonts/TTF/",
+        "LIBDIR/X11/fonts/truetype/",
+        "LIBDIR/X11/fonts/TrueType/",
+        "LIBDIR/X11/fonts/Type1/sun/",
+        "LIBDIR/X11/fonts/F3bitmaps/",
         "/usr/local/share/fonts/ttfonts",
         "/usr/share/fonts/default/Type1",
         "/usr/lib/openoffice/share/fonts/truetype",
         NULL
     };
-    
+
     /*
      * if a font server is running, set the font path to that
      *
@@ -262,19 +306,24 @@ static void add_font_path(GenerateOptions *gop, XConfigPtr config)
     ret = system("ps -C xfs 2>&1 > /dev/null");
 #endif
     if (WEXITSTATUS(ret) == 0) {
-        config->files->fontpath = "unix/:7100";
+        config->files->fontpath = xconfigStrdup("unix/:7100");
     } else {
+
+        /* get the X server libdir */
+
+        libdir = find_libdir(gop);
+
         for (i = 0; __font_paths[i]; i++) {
             path = xconfigStrdup(__font_paths[i]);
 
-            /* replace ROOT with the project root */
+            /* replace LIBDIR with libdir */
 
-            if (strncmp(path, "ROOT", 4) == 0) {
-                p = xconfigStrcat(gop->x_project_root, path + 4, NULL);
+            if (strncmp(path, "LIBDIR", 6) == 0) {
+                p = xconfigStrcat(libdir, path + 6, NULL);
                 free(path);
                 path = p;
             }
-        
+
             /* temporarily chop off any ":unscaled" appendage */
 
             p = strchr(path, ':');
@@ -290,7 +339,7 @@ static void add_font_path(GenerateOptions *gop, XConfigPtr config)
                 continue;
             }
             free(fonts_dir);
-        
+
             /* add the ":unscaled" back */
 
             if (p) *p = ':';
@@ -309,6 +358,10 @@ static void add_font_path(GenerateOptions *gop, XConfigPtr config)
                 config->files->fontpath = path;
             }
         }
+
+        /* free the libdir string */
+
+        free(libdir);
     }
 } /* add_font_path() */
 
@@ -322,81 +375,87 @@ static void add_modules(GenerateOptions *gop, XConfigPtr config)
 {
     XConfigLoadPtr l = NULL;
 
+    /*
+     * if the X server will automatically autoload GLX, then don't
+     * bother adding a modules section; it is difficult for
+     * nvidia-xconfig to know if modules like "type1" are present,
+     * anyway.
+     */
+
+    if (gop->autoloads_glx) return;
+
     config->modules = xconfigAlloc(sizeof(XConfigModuleRec));
-    
-    l = xconfigAddNewLoadDirective(l, "dbe", XCONFIG_LOAD_MODULE,
-                                   NULL, FALSE);
-    l = xconfigAddNewLoadDirective(l, "extmod", XCONFIG_LOAD_MODULE,
-                                   NULL, FALSE);
-    l = xconfigAddNewLoadDirective(l, "type1", XCONFIG_LOAD_MODULE,
-                                   NULL, FALSE);
+
+    xconfigAddNewLoadDirective(&l, xconfigStrdup("dbe"),
+                               XCONFIG_LOAD_MODULE, NULL, FALSE);
+    xconfigAddNewLoadDirective(&l, xconfigStrdup("extmod"),
+                               XCONFIG_LOAD_MODULE, NULL, FALSE);
+    xconfigAddNewLoadDirective(&l, xconfigStrdup("type1"),
+                               XCONFIG_LOAD_MODULE, NULL, FALSE);
 #if defined(NV_SUNOS)
-    l = xconfigAddNewLoadDirective(l, "IA", XCONFIG_LOAD_MODULE,
-                                   NULL, FALSE);
-    l = xconfigAddNewLoadDirective(l, "Xst", XCONFIG_LOAD_MODULE,
-                                   NULL, FALSE);
-    l = xconfigAddNewLoadDirective(l, "bitstream", XCONFIG_LOAD_MODULE,
-                                   NULL, FALSE);
+    xconfigAddNewLoadDirective(&l, xconfigStrdup("IA"),
+                               XCONFIG_LOAD_MODULE, NULL, FALSE);
+    xconfigAddNewLoadDirective(&l, xconfigStrdup("bitstream"),
+                               XCONFIG_LOAD_MODULE, NULL, FALSE);
+    xconfigAddNewLoadDirective(&l, xconfigStrdup("xtsol"),
+                               XCONFIG_LOAD_MODULE, NULL, FALSE);
 #else
-    l = xconfigAddNewLoadDirective(l, "freetype", XCONFIG_LOAD_MODULE,
-                                   NULL, FALSE);
+    xconfigAddNewLoadDirective(&l, xconfigStrdup("freetype"),
+                               XCONFIG_LOAD_MODULE, NULL, FALSE);
 #endif
-    l = xconfigAddNewLoadDirective(l, "glx", XCONFIG_LOAD_MODULE,
-                                   NULL, FALSE);
-    
+    xconfigAddNewLoadDirective(&l, xconfigStrdup("glx"),
+                               XCONFIG_LOAD_MODULE, NULL, FALSE);
+
     config->modules->loads = l;
-    
+
 } /* add_modules() */
 
 
 
 /*
- * add_monitor() -
+ * xconfigAddMonitor() -
  *
  * XXX pass EDID values into this...
  */
 
-static XConfigMonitorPtr
-add_monitor(XConfigPtr config, int count)
+XConfigMonitorPtr xconfigAddMonitor(XConfigPtr config, int count)
 {
     XConfigMonitorPtr monitor, m;
-    XConfigOptionPtr opt = NULL;
 
     /* XXX need to query resman for the EDID */
 
     monitor = xconfigAlloc(sizeof(XConfigMonitorRec));
-    
+
     monitor->identifier = xconfigAlloc(32);
     snprintf(monitor->identifier, 32, MONITOR_IDENTIFIER, count);
-    monitor->vendor = "Unknown";  /* XXX */
-    monitor->modelname = "Unknown"; /* XXX */
-    
+    monitor->vendor = xconfigStrdup("Unknown");  /* XXX */
+    monitor->modelname = xconfigStrdup("Unknown"); /* XXX */
+
     /* XXX check EDID for freq ranges */
 
     monitor->n_hsync = 1;
-    monitor->hsync[0].lo = 30.0;
-    monitor->hsync[0].hi = 110.0;
+    monitor->hsync[0].lo = 28.0;
+    monitor->hsync[0].hi = 33.0;
 
     monitor->n_vrefresh = 1;
-    monitor->vrefresh[0].lo = 50.0;
-    monitor->vrefresh[0].hi = 150.0;
+    monitor->vrefresh[0].lo = 43.0;
+    monitor->vrefresh[0].hi = 72.0;
 
-    opt = xconfigAddNewOption(opt, xconfigStrdup("DPMS"), NULL);
-
-    monitor->options = opt;
+    monitor->options = NULL;
+    xconfigAddNewOption(&monitor->options, "DPMS", NULL);
 
     /* append to the end of the monitor list */
-    
+
     if (!config->monitors) {
         config->monitors = monitor;
     } else {
         for (m = config->monitors; m->next; m = m->next);
         m->next = monitor;
     }
-    
+
     return monitor;
-   
-} /* add_monitor() */
+
+} /* xconfigAddMonitor() */
 
 
 
@@ -422,47 +481,41 @@ add_device(XConfigPtr config, int bus, int slot, char *boardname, int count)
     }
 
     if (boardname) device->board = xconfigStrdup(boardname);
-    
+
     device->chipid = -1;
     device->chiprev = -1;
     device->irq = -1;
     device->screen = -1;
-    
+
     /* append to the end of the device list */
-    
+
     if (!config->devices) {
         config->devices = device;
     } else {
         for (d = config->devices; d->next; d = d->next);
         d->next = device;
     }
-    
+
     return device;
-    
+
 } /* add_device() */
 
 
 
-static XConfigDisplayPtr add_display(int depth)
+void xconfigAddDisplay(XConfigDisplayPtr *pHead, const int depth)
 {
     XConfigDisplayPtr display;
-    XConfigModePtr mode = NULL;
-    
-    mode = xconfigAddMode(mode, "640x480");
-    mode = xconfigAddMode(mode, "800x600");
-    mode = xconfigAddMode(mode, "1024x768");
-    mode = xconfigAddMode(mode, "1280x1024");
-    mode = xconfigAddMode(mode, "1600x1200");
-    
+
     display = xconfigAlloc(sizeof(XConfigDisplayRec));
     display->depth = depth;
-    display->modes = mode;
+    display->modes = NULL;
     display->frameX0 = -1;
     display->frameY0 = -1;
     display->black.red = -1;
     display->white.red = -1;
-    
-    return display;
+
+    display->next = *pHead;
+    *pHead = display;
 }
 
 
@@ -476,33 +529,33 @@ static void add_layout(GenerateOptions *gop, XConfigPtr config)
     XConfigLayoutPtr layout;
     XConfigAdjacencyPtr adj;
     XConfigScreenPtr screen;
-    
+
     /* assume 1 X screen */
 
     screen = xconfigGenerateAddScreen(config, -1, -1, NULL, 0);
-    
+
     /* create layout */
 
     layout = xconfigAlloc(sizeof(XConfigLayoutRec));
-    
-    layout->identifier = "Layout0";
-    
+
+    layout->identifier = xconfigStrdup("Layout0");
+
     adj = xconfigAlloc(sizeof(XConfigAdjacencyRec));
 
     adj->scrnum = 0;
     adj->screen = screen;
     adj->screen_name = xconfigStrdup(screen->identifier);
-    
+
     layout->adjacencies = adj;
-    
+
     xconfigGenerateAssignScreenAdjacencies(layout);
-    
+
     add_inputref(config, layout, MOUSE_IDENTIFER, "CorePointer");
     add_inputref(config, layout, KEYBOARD_IDENTIFER, "CoreKeyboard");
-    
+
     layout->next = config->layouts;
     config->layouts = layout;
-    
+
 } /* add_layout() */
 
 
@@ -517,10 +570,10 @@ static void add_inputref(XConfigPtr config, XConfigLayoutPtr layout,
     XConfigInputrefPtr inputRef;
 
     inputRef = xconfigAlloc(sizeof(XConfigInputrefRec));
-    inputRef->input_name = name;
+    inputRef->input_name = xconfigStrdup(name);
     inputRef->input = xconfigFindInput(inputRef->input_name, config->inputs);
-    inputRef->options =
-        xconfigAddNewOption(NULL, xconfigStrdup(coreKeyword), NULL);
+    inputRef->options = NULL;
+    xconfigAddNewOption(&inputRef->options, coreKeyword, NULL);
     inputRef->next = layout->inputs;
     layout->inputs = inputRef;
 
@@ -699,7 +752,7 @@ static const MouseEntry *find_closest_mouse_entry(const char *device,
 {
     int i;
     int emulate3 = FALSE;
-    
+
     /*
      * translate the emulate3 string into a boolean we can use below
      * for comparison
@@ -715,20 +768,20 @@ static const MouseEntry *find_closest_mouse_entry(const char *device,
     /*
      * skip the "/dev/" part of the device filename
      */
-    
+
     if (device && (strncmp(device, "/dev/", 5) == 0)) {
         device += 5; /* strlen("/dev/") */
     }
-    
+
     for (i = 0; __mice[i].name; i++) {
         if ((device) && (strcmp(device, __mice[i].device) != 0)) continue;
         if ((proto) && (strcasecmp(proto, __mice[i].Xproto)) != 0) continue;
         if ((emulate3_str) && (emulate3 != __mice[i].emulate3)) continue;
         return &__mice[i];
     }
-    
+
     return NULL;
-    
+
 } /* find_closest_mouse_entry() */
 
 
@@ -744,20 +797,33 @@ static char *find_config_entry(const char *filename, const char *keyword)
     int fd = -1;
     char *data = NULL;
     char *value = NULL;
+    char *buf = NULL;
     char *tmp, *start, *c, *end;
     struct stat stat_buf;
     size_t len;
-    
+
     if ((fd = open(filename, O_RDONLY)) == -1) goto done;
-    
+
     if (fstat(fd, &stat_buf) == -1) goto done;
-    
+
     if ((data = mmap(0, stat_buf.st_size, PROT_READ, MAP_SHARED,
                      fd, 0)) == (void *) -1) goto done;
-    
+
+    /*
+     * create a sysmem copy of the buffer, so that we can explicitly
+     * NULL terminate it
+     */
+
+    buf = malloc(stat_buf.st_size + 1);
+
+    if (!buf) goto done;
+
+    memcpy(buf, data, stat_buf.st_size);
+    buf[stat_buf.st_size] = '\0';
+
     /* search for the keyword */
-    
-    start = data;
+
+    start = buf;
 
     while (TRUE) {
         tmp = strstr(start, keyword);
@@ -771,7 +837,7 @@ static char *find_config_entry(const char *filename, const char *keyword)
 
         c = tmp;
         while ((c >= start) && (*c != '\n') && (*c != '#')) c--;
-        
+
         if (*c == '#') {
             /* keyword was commented out... search again */
             start = tmp+1;
@@ -788,9 +854,9 @@ static char *find_config_entry(const char *filename, const char *keyword)
     /* there must be something between the start and the end */
 
     if (start == end) goto done;
-    
+
     /* take what is between as the value */
-    
+
     len = end - start;
     value = xconfigAlloc(len + 1);
     strncpy(value, start, len);
@@ -805,14 +871,15 @@ static char *find_config_entry(const char *filename, const char *keyword)
         free(value);
         value = tmp;
     }
-    
+
  done:
 
+    if (buf) free(buf);
     if (data) munmap(data, stat_buf.st_size);
     if (fd != -1) close(fd);
 
     return value;
-    
+
 } /* find_config_entry() */
 
 
@@ -824,13 +891,13 @@ static char *find_config_entry(const char *filename, const char *keyword)
 void xconfigGeneratePrintPossibleMice(void)
 {
     int i;
-    
+
     printf("%-25s%-35s\n\n", "Short Name", "Name");
-    
+
     for (i = 0; __mice[i].name; i++) {
         printf("%-25s%-35s\n", __mice[i].shortname, __mice[i].name);
     }
-    
+
     printf("\n");
 
 } /* xconfigGeneratePrintPossibleMice() */
@@ -838,7 +905,7 @@ void xconfigGeneratePrintPossibleMice(void)
 
 
 /*
- * add_mouse() - determine the mouse type, and then add an
+ * xconfigAddMouse() - determine the mouse type, and then add an
  * XConfigInputRec with the appropriate options.
  *
  * - if the user specified on the commandline, use that
@@ -853,15 +920,14 @@ void xconfigGeneratePrintPossibleMice(void)
  * - default to "auto" on /dev/mouse
  */
 
-static int add_mouse(GenerateOptions *gop, XConfigPtr config)
+int xconfigAddMouse(GenerateOptions *gop, XConfigPtr config)
 {
     const MouseEntry *entry = NULL;
     XConfigInputPtr input;
-    XConfigOptionPtr opt = NULL;
     char *device_path, *comment = "default";
-    
+
     /* if the user specified on the commandline, use that */
-    
+
     if (gop->mouse) {
         entry = find_mouse_entry(gop->mouse);
         if (entry) {
@@ -871,19 +937,19 @@ static int add_mouse(GenerateOptions *gop, XConfigPtr config)
                             gop->mouse);
         }
     }
-    
+
     /*
      * if /etc/sysconfig/mouse exists, and contains valid data, use
      * that
      */
-    
+
     if (!entry) {
         char *protocol, *device, *emulate3;
-        
+
         device = find_config_entry("/etc/sysconfig/mouse", "DEVICE=");
         protocol = find_config_entry("/etc/sysconfig/mouse", "XMOUSETYPE=");
         emulate3 = find_config_entry("/etc/sysconfig/mouse", "XEMU3=");
-        
+
         if (device || protocol || emulate3) {
             entry = find_closest_mouse_entry(device, protocol, emulate3);
             if (entry) {
@@ -896,10 +962,10 @@ static int add_mouse(GenerateOptions *gop, XConfigPtr config)
 
     if (!entry) {
         char *protocol, *device;
-        
+
         protocol = find_config_entry("/etc/conf.d/gpm", "MOUSE=");
         device = find_config_entry("/etc/conf.d/gpm", "MOUSEDEV=");
-        
+
         if (protocol && device) {
             MouseEntry *e = xconfigAlloc(sizeof(MouseEntry));
             e->shortname = "custom";
@@ -912,7 +978,7 @@ static int add_mouse(GenerateOptions *gop, XConfigPtr config)
             comment = "data in \"/etc/conf.d/gpm\"";
         }
     }
-    
+
     /*
      * XXX we could try to infer the settings from the commandline
      * options gpm is using
@@ -921,13 +987,16 @@ static int add_mouse(GenerateOptions *gop, XConfigPtr config)
     if (!entry) {
         /* XXX implement me */
     }
-    
+
     /* at this point, we must have a mouse entry */
-    
+
     if (!entry) {
         MouseEntry *e = xconfigAlloc(sizeof(MouseEntry));
         e->Xproto = "auto";
 
+#if defined(NV_BSD)
+        e->device = "sysmouse";
+#else
         if (access("/dev/psaux", F_OK) == 0) {
             e->device = "psaux";
         } else if (access("/dev/input/mice", F_OK) == 0) {
@@ -935,7 +1004,7 @@ static int add_mouse(GenerateOptions *gop, XConfigPtr config)
         } else {
             e->device = "mouse";
         }
-        
+#endif
         e->emulate3 = FALSE;
         entry = e;
     }
@@ -943,35 +1012,35 @@ static int add_mouse(GenerateOptions *gop, XConfigPtr config)
     /* add a new mouse input section */
 
     input = xconfigAlloc(sizeof(XConfigInputRec));
-    
+
     input->comment = xconfigStrcat("    # generated from ",
                                    comment, "\n", NULL);
-    input->identifier = "Mouse0";
-    input->driver = "mouse";
+    input->identifier = xconfigStrdup("Mouse0");
+    input->driver = xconfigStrdup("mouse");
 
     device_path = xconfigStrcat("/dev/", entry->device, NULL);
 
-    opt = xconfigAddNewOption(opt, xconfigStrdup("Protocol"), entry->Xproto);
-    opt = xconfigAddNewOption(opt, xconfigStrdup("Device"), device_path);
-    opt = xconfigAddNewOption(opt, xconfigStrdup("Emulate3Buttons"),
-                              entry->emulate3 ?
-                              xconfigStrdup("yes") : xconfigStrdup("no"));
-    
+    input->options = NULL;
+    xconfigAddNewOption(&input->options, "Protocol", entry->Xproto);
+    xconfigAddNewOption(&input->options, "Device", device_path);
+    xconfigAddNewOption(&input->options, "Emulate3Buttons",
+                              (entry->emulate3 ? "yes" : "no"));
+    TEST_FREE(device_path);
+
+
     /*
      * This will make wheel mice work, and non-wheel mice should
      * ignore ZAxisMapping
      */
 
-    opt = xconfigAddNewOption(opt, "ZAxisMapping", "4 5");
-    
-    input->options = opt;
-    
+    xconfigAddNewOption(&input->options, "ZAxisMapping", "4 5");
+
     input->next = config->inputs;
     config->inputs = input;
-    
+
     return TRUE;
-    
-} /* add_mouse() */
+
+} /* xconfigAddMouse() */
 
 
 
@@ -1000,23 +1069,23 @@ typedef struct {
  *
  *
  * keyboard_models.py - keyboard model list
- * 
+ *
  * Brent Fox <bfox@redhat.com>
  * Mike Fulbright <msf@redhat.com>
  * Jeremy Katz <katzj@redhat.com>
- * 
+ *
  * Copyright 2002 Red Hat, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
@@ -1024,7 +1093,7 @@ typedef struct {
  */
 
 static const KeyboardEntry __keyboards[] = {
-    
+
     /* keytable               name                              layout         model  variant options */
 
     { "be-latin1",            "Belgian (be-latin1)",            "be",          "pc105", NULL, NULL },
@@ -1117,13 +1186,13 @@ static const KeyboardEntry *find_keyboard_entry(char *value)
 void xconfigGeneratePrintPossibleKeyboards(void)
 {
     int i;
-    
+
     printf("%-25s%-35s\n\n", "Short Name", "Name");
 
     for (i = 0; __keyboards[i].name; i++) {
         printf("%-25s%-35s\n", __keyboards[i].keytable, __keyboards[i].name);
     }
-    
+
     printf("\n");
 
 } /* xconfigGeneratePrintPossibleKeyboards() */
@@ -1131,7 +1200,7 @@ void xconfigGeneratePrintPossibleKeyboards(void)
 
 
 /*
- * add_keyboard() - determine the keyboard type, and then add an
+ * xconfigAddKeyboard() - determine the keyboard type, and then add an
  * XConfigInputRec with the appropriate options.
  *
  * How to detect the keyboard:
@@ -1142,14 +1211,13 @@ void xconfigGeneratePrintPossibleKeyboards(void)
  *   KEYTABLE entry, use that
  */
 
-static int add_keyboard(GenerateOptions *gop, XConfigPtr config)
+int xconfigAddKeyboard(GenerateOptions *gop, XConfigPtr config)
 {
     char *value, *comment = "default";
     const KeyboardEntry *entry = NULL;
-    
+
     XConfigInputPtr input;
-    XConfigOptionPtr opt = NULL;
-    
+
     /*
      * if the user specified on the command line, use that
      */
@@ -1163,7 +1231,7 @@ static int add_keyboard(GenerateOptions *gop, XConfigPtr config)
                             gop->keyboard);
         }
     }
-    
+
     /*
      * if /etc/sysconfig/keyboard exists, and contains a valid
      * KEYTABLE entry, use that
@@ -1181,12 +1249,12 @@ static int add_keyboard(GenerateOptions *gop, XConfigPtr config)
     }
 
     /* add a new keyboard input section */
-    
+
     input = xconfigAlloc(sizeof(XConfigInputRec));
-    
+
     input->comment = xconfigStrcat("    # generated from ",
                                    comment, "\n", NULL);
-    input->identifier = "Keyboard0";
+    input->identifier = xconfigStrdup("Keyboard0");
 
     /*
      * determine which keyboard driver should be used (either "kbd" or
@@ -1195,44 +1263,287 @@ static int add_keyboard(GenerateOptions *gop, XConfigPtr config)
      * otherwise, use "keyboard".
      * On Solaris, use the default "keyboard"
      */
-    
+
     if (gop->keyboard_driver) {
         input->driver = gop->keyboard_driver;
     } else {
 #if defined(NV_SUNOS) || defined(NV_BSD)
-        input->driver = "keyboard";
+        input->driver = xconfigStrdup("keyboard");
 #else
         if (gop->xserver == X_IS_XORG) {
-            input->driver = "kbd";
+            input->driver = xconfigStrdup("kbd");
         } else {
-            input->driver = "keyboard";
+            input->driver = xconfigStrdup("keyboard");
         }
 #endif
     }
-    
+
     /*
      * set additional keyboard options, based on the Keyboard table
      * entry we found above
      */
 
-    if (entry && entry->layout)
-        opt = xconfigAddNewOption(opt,
-                                  xconfigStrdup("XkbLayout"), entry->layout);
-    if (entry && entry->model)
-        opt = xconfigAddNewOption(opt,
-                                  xconfigStrdup("XkbModel"), entry->model);
-    if (entry && entry->variant)
-        opt = xconfigAddNewOption(opt,
-                                  xconfigStrdup("XkbVariant"), entry->variant);
-    if (entry && entry->options)
-        opt = xconfigAddNewOption(opt,
-                                  xconfigStrdup("XkbOptions"), entry->options);
+    input->options = NULL;
+
+    if (entry) {
+        if (entry->layout)
+            xconfigAddNewOption(&input->options, "XkbLayout", entry->layout);
+        if (entry->model)
+            xconfigAddNewOption(&input->options, "XkbModel", entry->model);
+        if (entry->variant)
+            xconfigAddNewOption(&input->options, "XkbVariant", entry->variant);
+        if (entry->options)
+            xconfigAddNewOption(&input->options, "XkbOptions", entry->options);
+    }
 
-    input->options = opt;
-    
     input->next = config->inputs;
     config->inputs = input;
 
     return TRUE;
-    
-} /* add_keyboard() */
+
+} /* xconfigAddKeyboard() */
+
+
+
+/*
+ * xconfigGetDefaultProjectRoot() - scan some common directories for the X
+ * project root.
+ *
+ * Users of this information should be careful to account for the
+ * modular layout.
+ */
+
+static char *xconfigGetDefaultProjectRoot(void)
+{
+    char *paths[] = { "/usr/X11R6", "/usr/X11", NULL };
+    struct stat stat_buf;
+    int i;
+
+    for (i = 0; paths[i]; i++) {
+
+        if (stat(paths[i], &stat_buf) == -1) {
+            continue;
+        }
+
+        if (S_ISDIR(stat_buf.st_mode)) {
+            return paths[i];
+        }
+    }
+
+    /* default to "/usr/X11R6", I guess */
+
+    return paths[0];
+
+} /* xconfigGetDefaultProjectRoot() */
+
+
+
+
+/*
+ * get_xserver_information() - parse the versionString (from `X
+ * -version`) and assign relevant information that we infer from the X
+ * server version.
+ *
+ * Note: this implementation should be shared with nvidia-installer
+ */
+
+static int get_xserver_information(const char *versionString,
+                                   int *isXorg,
+                                   int *isModular,
+                                   int *autoloadsGLX,
+                                   int *supportsExtensionSection)
+{
+#define XSERVER_VERSION_FORMAT_1 "X Window System Version"
+#define XSERVER_VERSION_FORMAT_2 "X.Org X Server"
+
+    int major, minor, found;
+    const char *ptr;
+
+    /* check if this is an XFree86 X server */
+
+    if (strstr(versionString, "XFree86 Version")) {
+        *isXorg = FALSE;
+        *isModular = FALSE;
+        *autoloadsGLX = FALSE;
+        *supportsExtensionSection = FALSE;
+        return TRUE;
+    }
+
+    /* this must be an X.Org X server */
+
+    *isXorg = TRUE;
+
+    /* attempt to parse the major.minor version out of the string */
+
+    found = FALSE;
+
+    if (((ptr = strstr(versionString, XSERVER_VERSION_FORMAT_1)) != NULL) &&
+        (sscanf(ptr, XSERVER_VERSION_FORMAT_1 " %d.%d", &major, &minor) == 2)) {
+        found = TRUE;
+    }
+
+    if (!found &&
+        ((ptr = strstr(versionString, XSERVER_VERSION_FORMAT_2)) != NULL) &&
+        (sscanf(ptr, XSERVER_VERSION_FORMAT_2 " %d.%d", &major, &minor) == 2)) {
+        found = TRUE;
+    }
+
+    /* if we can't parse the version, give up */
+
+    if (!found) return FALSE;
+
+    /*
+     * isModular: X.Org X11R6.x X servers are monolithic, all others
+     * are modular
+     */
+
+    if (major == 6) {
+        *isModular = FALSE;
+    } else {
+        *isModular = TRUE;
+    }
+
+    /*
+     * supportsExtensionSection: support for the "Extension" xorg.conf
+     * section was added between X.Org 6.7 and 6.8.  To account for
+     * the X server version wrap, it is easier to check for X servers
+     * that do not support the Extension section: 6.x (x < 8) X
+     * servers.
+     */
+
+    if ((major == 6) && (minor < 8)) {
+        *supportsExtensionSection = FALSE;
+    } else {
+        *supportsExtensionSection = TRUE;
+    }
+
+    /*
+     * support for autoloading GLX was added in X.Org 1.5.  To account
+     * for the X server version wrap, it is easier to check for X
+     * servers that do not support GLX autoloading: 6.x, 7.x, or < 1.5
+     * X servers.
+     */
+
+    if ((major == 6) || (major == 7) || ((major == 1) && (minor < 5))) {
+        *autoloadsGLX = FALSE;
+    } else {
+        *autoloadsGLX = TRUE;
+    }
+
+    return TRUE;
+
+} /* get_xserver_information() */
+
+
+
+/*
+ * xconfigGetXServerInUse() - try to determine which X server is in use
+ * (XFree86, Xorg); also determine if the X server supports the
+ * Extension section of the X config file; support for the "Extension"
+ * section was added between X.Org 6.7 and 6.8.
+ *
+ * Some of the parsing here mimics what is done in the
+ * check_for_modular_xorg() function in nvidia-installer
+ */
+
+#define NV_LINE_LEN 1024
+#define EXTRA_PATH "/bin:/usr/bin:/sbin:/usr/sbin:/usr/X11R6/bin:/usr/bin/X11"
+#if defined(NV_SUNOS)
+#define XSERVER_BIN_NAME "Xorg"
+#else
+#define XSERVER_BIN_NAME "X"
+#endif
+
+
+void xconfigGetXServerInUse(GenerateOptions *gop)
+{
+    FILE *stream = NULL;
+    int xserver = -1;
+    int isXorg;
+    int dummy, len, found;
+    char *cmd, *ptr, *ret;
+
+    gop->supports_extension_section = FALSE;
+    gop->autoloads_glx = FALSE;
+
+    /* run `X -version` with a PATH that hopefully includes the X binary */
+
+    cmd = xconfigStrcat("PATH=", gop->x_project_root, ":",
+                        EXTRA_PATH, ":$PATH ", XSERVER_BIN_NAME,
+                        " -version 2>&1", NULL);
+
+    if ((stream = popen(cmd, "r"))) {
+        char buf[NV_LINE_LEN];
+
+        /* read in as much of the input as we can fit into the buffer */
+
+        ptr = buf;
+
+        do {
+            len = NV_LINE_LEN - (ptr - buf) - 1;
+            ret = fgets(ptr, len, stream);
+            ptr = strchr(ptr, '\0');
+        } while ((ret != NULL) && (len > 1));
+
+        /*
+         * process the `X -version` output to infer relevant
+         * information from this X server
+         */
+
+        found = get_xserver_information(buf,
+                                        &isXorg,
+                                        &dummy, /* isModular */
+                                        &gop->autoloads_glx,
+                                        &gop->supports_extension_section);
+
+        if (found) {
+            if (isXorg) {
+                xserver = X_IS_XORG;
+            } else {
+                xserver = X_IS_XF86;
+            }
+        } else {
+            xconfigErrorMsg(WarnMsg, "Unable to parse X.Org version string.");
+        }
+    }
+    /* Close the popen()'ed stream. */
+    pclose(stream);
+    free(cmd);
+
+    if (xserver == -1) {
+        char *xorgpath;
+
+        xorgpath = xconfigStrcat(gop->x_project_root, "/bin/Xorg", NULL);
+        if (access(xorgpath, F_OK)==0) {
+            xserver = X_IS_XORG;
+        } else {
+            xserver = X_IS_XF86;
+        }
+        free(xorgpath);
+    }
+
+    gop->xserver=xserver;
+
+} /* xconfigGetXServerInUse() */
+
+
+
+/*
+ * xconfigGenerateLoadDefaultOptions - initialize a GenerateOptions
+ * structure with default values by peeking at the file system.
+ */
+
+void xconfigGenerateLoadDefaultOptions(GenerateOptions *gop)
+{
+    memset(gop, 0, sizeof(GenerateOptions));
+
+    gop->x_project_root = xconfigGetDefaultProjectRoot();
+
+    /* XXX What to default the following to?
+       gop->xserver
+       gop->keyboard
+       gop->mouse
+       gop->keyboard_driver
+     */
+
+} /* xconfigGenerateLoadDefaultOptions() */
