@@ -79,7 +79,7 @@
 #endif  // defined(__linux__)
 
 // Unix variations:  SunOS
-#if !defined(NV_SUNOS) && defined(__sun__) || defined(__sun)
+#if !defined(NV_SUNOS) && (defined(__sun__) || defined(__sun))
 #   define NV_SUNOS
 #endif // defined(__sun__)
 
@@ -148,7 +148,6 @@ typedef struct __xconfigoptionrec {
     struct __xconfigoptionrec *next;
     char *name;
     char *val;
-    int   used;
     char *comment;
 } XConfigOptionRec, *XConfigOptionPtr;
 
@@ -170,7 +169,7 @@ typedef struct {
 /* Values for load_type */
 #define XCONFIG_LOAD_MODULE    0
 #define XCONFIG_LOAD_DRIVER    1
-
+#define XCONFIG_DISABLE_MODULE 2
 
 
 /*
@@ -187,6 +186,7 @@ typedef struct __xconfigloadrec {
 
 typedef struct {
     XConfigLoadPtr  loads;
+    XConfigLoadPtr  disables;
     char           *comment;
 } XConfigModuleRec, *XConfigModulePtr;
 
@@ -217,7 +217,7 @@ typedef struct {
 typedef struct __xconfigconfmodelinerec {
     struct __xconfigconfmodelinerec *next;
     char *identifier;
-    int clock;
+    char *clock; /* stored in MHz */
     int hdisplay;
     int hsyncstart;
     int hsyncend;
@@ -578,16 +578,38 @@ typedef struct {
 } XConfigSymTabRec, *XConfigSymTabPtr;
 
 
+/*
+ * data structure containing options; used during generation of X
+ * config, and when sanitizing an existing config
+ */
+
+#define X_IS_XF86 0
+#define X_IS_XORG 1
+
+typedef struct {
+    int   xserver;
+    char *x_project_root;
+    char *keyboard;
+    char *mouse;
+    char *keyboard_driver;
+
+    int supports_extension_section;
+    int autoloads_glx;
+
+} GenerateOptions;
+
 
 /*
  * Functions for open, reading, and writing XConfig files.
  */
 const char *xconfigOpenConfigFile(const char *, const char *);
-XConfigError xconfigReadConfigFile(const char *, XConfigPtr *);
+XConfigError xconfigReadConfigFile(XConfigPtr *);
+int xconfigSanitizeConfig(XConfigPtr p, const char *screenName,
+                          GenerateOptions *gop);
 void xconfigCloseConfigFile(void);
 int xconfigWriteConfigFile(const char *, XConfigPtr);
 
-void xconfigFreeConfig(XConfigPtr p);
+void xconfigFreeConfig(XConfigPtr *p);
 
 /*
  * Functions for searching for entries in lists
@@ -600,6 +622,7 @@ XConfigModesPtr    xconfigFindModes(const char *ident, XConfigModesPtr p);
 XConfigModeLinePtr xconfigFindModeLine(const char *ident,
                                        XConfigModeLinePtr p);
 XConfigScreenPtr   xconfigFindScreen(const char *ident, XConfigScreenPtr p);
+XConfigModePtr     xconfigFindMode(const char *name, XConfigModePtr p);
 XConfigInputPtr    xconfigFindInput(const char *ident, XConfigInputPtr p);
 XConfigInputPtr    xconfigFindInputByDriver(const char *driver,
                                             XConfigInputPtr p);
@@ -611,71 +634,66 @@ XConfigVideoAdaptorPtr xconfigFindVideoAdaptor(const char *ident,
  * Functions for freeing lists
  */
 
-void xconfigFreeDeviceList(XConfigDevicePtr ptr);
-void xconfigFreeFiles(XConfigFilesPtr p);
-void xconfigFreeFlags(XConfigFlagsPtr flags);
-void xconfigFreeInputList(XConfigInputPtr ptr);
-void xconfigFreeLayoutList(XConfigLayoutPtr ptr);
-void xconfigFreeAdjacencyList(XConfigAdjacencyPtr ptr);
-void xconfigFreeInputrefList(XConfigInputrefPtr ptr);
-void xconfigFreeModules(XConfigModulePtr ptr);
-void xconfigFreeMonitorList(XConfigMonitorPtr ptr);
-void xconfigFreeModesList(XConfigModesPtr ptr);
-void xconfigFreeModeLineList(XConfigModeLinePtr ptr);
-void xconfigFreeScreenList(XConfigScreenPtr ptr);
-void xconfigFreeAdaptorLinkList(XConfigAdaptorLinkPtr ptr);
-void xconfigFreeDisplayList(XConfigDisplayPtr ptr);
-void xconfigFreeModeList(XConfigModePtr ptr);
-void xconfigFreeVendorList(XConfigVendorPtr p);
-void xconfigFreeVendorSubList(XConfigVendSubPtr ptr);
-void xconfigFreeVideoAdaptorList(XConfigVideoAdaptorPtr ptr);
-void xconfigFreeVideoPortList(XConfigVideoPortPtr ptr);
-void xconfigFreeBuffersList (XConfigBuffersPtr ptr);
-void xconfigFreeDRI(XConfigDRIPtr ptr);
-void xconfigFreeExtensions(XConfigExtensionsPtr ptr);
+void xconfigFreeDeviceList(XConfigDevicePtr *ptr);
+void xconfigFreeFiles(XConfigFilesPtr *p);
+void xconfigFreeFlags(XConfigFlagsPtr *flags);
+void xconfigFreeInputList(XConfigInputPtr *ptr);
+void xconfigFreeLayoutList(XConfigLayoutPtr *ptr);
+void xconfigFreeAdjacencyList(XConfigAdjacencyPtr *ptr);
+void xconfigFreeInputrefList(XConfigInputrefPtr *ptr);
+void xconfigFreeModules(XConfigModulePtr *ptr);
+void xconfigFreeMonitorList(XConfigMonitorPtr *ptr);
+void xconfigFreeModesList(XConfigModesPtr *ptr);
+void xconfigFreeModeLineList(XConfigModeLinePtr *ptr);
+void xconfigFreeOptionList(XConfigOptionPtr *opt);
+void xconfigFreeScreenList(XConfigScreenPtr *ptr);
+void xconfigFreeAdaptorLinkList(XConfigAdaptorLinkPtr *ptr);
+void xconfigFreeDisplayList(XConfigDisplayPtr *ptr);
+void xconfigFreeModeList(XConfigModePtr *ptr);
+void xconfigFreeVendorList(XConfigVendorPtr *p);
+void xconfigFreeVendorSubList(XConfigVendSubPtr *ptr);
+void xconfigFreeVideoAdaptorList(XConfigVideoAdaptorPtr *ptr);
+void xconfigFreeVideoPortList(XConfigVideoPortPtr *ptr);
+void xconfigFreeBuffersList (XConfigBuffersPtr *ptr);
+void xconfigFreeDRI(XConfigDRIPtr *ptr);
+void xconfigFreeExtensions(XConfigExtensionsPtr *ptr);
+void xconfigFreeModesLinkList(XConfigModesLinkPtr *ptr);
 
 
-/*
- * check (and update, if necessary) the inputs in the specified layout
- * section
- */
-
-int xconfigCheckCoreInputDevices(XConfigPtr config, XConfigLayoutPtr layout);
 
 /*
  * item/list manipulation
  */
 
-GenericListPtr xconfigAddListItem(GenericListPtr head, GenericListPtr c_new);
+void xconfigAddListItem(GenericListPtr *pHead, GenericListPtr c_new);
+void xconfigRemoveListItem(GenericListPtr *pHead, GenericListPtr item);
 int xconfigItemNotSublist(GenericListPtr list_1, GenericListPtr list_2);
 char *xconfigAddComment(char *cur, char *add);
-XConfigLoadPtr xconfigAddNewLoadDirective(XConfigLoadPtr head,
-                                          char *name, int type,
-                                          XConfigOptionPtr opts, int do_token);
-XConfigLoadPtr xconfigRemoveLoadDirective(XConfigLoadPtr head,
-                                          XConfigLoadPtr load);
+void xconfigAddNewLoadDirective(XConfigLoadPtr *pHead,
+                                char *name, int type,
+                                XConfigOptionPtr opts, int do_token);
+void xconfigRemoveLoadDirective(XConfigLoadPtr *pHead, XConfigLoadPtr load);
 
 /*
  * Functions for manipulating Options
  */
 
-XConfigOptionPtr xconfigAddNewOption(XConfigOptionPtr head,
-                                     char *name, char *val);
-XConfigOptionPtr xconfigRemoveOption(XConfigOptionPtr list,
-                                     XConfigOptionPtr opt);
+void xconfigAddNewOption(XConfigOptionPtr *pHead,
+                         const char *name, const char *val);
+void xconfigRemoveOption(XConfigOptionPtr *pHead, XConfigOptionPtr opt);
+void xconfigRemoveNamedOption(XConfigOptionPtr *head, const char *name,
+                              char **comments);
+
 XConfigOptionPtr xconfigOptionListDup(XConfigOptionPtr opt);
-void             xconfigOptionListFree(XConfigOptionPtr opt);
 char            *xconfigOptionName(XConfigOptionPtr opt);
 char            *xconfigOptionValue(XConfigOptionPtr opt);
-XConfigOptionPtr xconfigNewOption(char *name, char *value);
+XConfigOptionPtr xconfigNewOption(const char *name, const char *value);
 XConfigOptionPtr xconfigNextOption(XConfigOptionPtr list);
 XConfigOptionPtr xconfigFindOption(XConfigOptionPtr list, const char *name);
 char            *xconfigFindOptionValue(XConfigOptionPtr list,
                                         const char *name);
 int              xconfigFindOptionBoolean (XConfigOptionPtr,
-                                           const char *, int);
-XConfigOptionPtr xconfigOptionListCreate(const char **options,
-                                         int count, int used);
+                                           const char *name);
 XConfigOptionPtr xconfigOptionListMerge(XConfigOptionPtr head,
                                         XConfigOptionPtr tail);
 
@@ -686,30 +704,17 @@ XConfigOptionPtr xconfigOptionListMerge(XConfigOptionPtr head,
 char *xconfigStrdup(const char *s);
 char *xconfigStrcat(const char *str, ...);
 int xconfigNameCompare(const char *s1, const char *s2);
+int xconfigModelineCompare(XConfigModeLinePtr m1, XConfigModeLinePtr m2);
 char *xconfigULongToString(unsigned long i);
-void xconfigDebugListOptions(XConfigOptionPtr);
 XConfigOptionPtr xconfigParseOption(XConfigOptionPtr head);
 void xconfigPrintOptionList(FILE *fp, XConfigOptionPtr list, int tabs);
 int xconfigParsePciBusString(const char *busID,
                              int *bus, int *device, int *func);
 
-XConfigModePtr
-xconfigAddMode(XConfigModePtr head, const char *name);
+void xconfigAddDisplay(XConfigDisplayPtr *pHead, const int depth);
 
-XConfigModePtr
-xconfigRemoveMode(XConfigModePtr head, const char *name);
-
-
-#define X_IS_XF86 0
-#define X_IS_XORG 1
-
-typedef struct {
-    int   xserver;
-    char *x_project_root;
-    char *keyboard;
-    char *mouse;
-    char *keyboard_driver;
-} GenerateOptions;
+void xconfigAddMode(XConfigModePtr *pHead, const char *name);
+void xconfigRemoveMode(XConfigModePtr *pHead, const char *name);
 
 XConfigPtr xconfigGenerate(GenerateOptions *gop);
 
@@ -720,5 +725,26 @@ void xconfigGenerateAssignScreenAdjacencies(XConfigLayoutPtr layout);
 
 void xconfigGeneratePrintPossibleMice(void);
 void xconfigGeneratePrintPossibleKeyboards(void);
+void xconfigGenerateLoadDefaultOptions(GenerateOptions *gop);
+
+void xconfigGetXServerInUse(GenerateOptions *gop);
+
+
+/*
+ * check (and update, if necessary) the inputs in the specified layout
+ * section
+ */
+
+int xconfigCheckCoreInputDevices(GenerateOptions *gop,
+                                 XConfigPtr config, XConfigLayoutPtr layout);
+
+
+/*
+ * X config tools
+ */
+
+int xconfigMergeConfigs(XConfigPtr dstConfig, XConfigPtr srcConfig);
+
+
 
 #endif /* _xf86Parser_h_ */
