@@ -35,10 +35,29 @@
 #include "configProcs.h"
 
 
-static int update_device(XConfigPtr config, XConfigDevicePtr device);
+static void ensure_module_loaded(XConfigPtr config, char *name);
+static int update_device(Options *op, XConfigPtr config, XConfigDevicePtr device);
 static void update_depth(Options *op, XConfigScreenPtr screen);
 static void update_display(Options *op, XConfigScreenPtr screen);
 
+/*
+ * ensure_module_loaded() - make sure the given module is present
+ */
+
+static void ensure_module_loaded(XConfigPtr config, char *name) {
+    XConfigLoadPtr load;
+    int found = FALSE;
+
+    for (load = config->modules->loads; load && !found; load = load->next) {
+        if (xconfigNameCompare(name, load->name) == 0) found = TRUE;
+    }
+
+    if (!found) {
+        xconfigAddNewLoadDirective(&config->modules->loads,
+                                   name, XCONFIG_LOAD_MODULE,
+                                   NULL, FALSE);
+    }
+} /* ensure_module_loaded */
 
 /*
  * update_modules() - make sure the glx module is present, and remove
@@ -48,7 +67,6 @@ static void update_display(Options *op, XConfigScreenPtr screen);
 int update_modules(XConfigPtr config)
 {
     XConfigLoadPtr load, next;
-    int found;
 
     /*
      * Return early if the original X configuration file lacked a
@@ -62,19 +80,11 @@ int update_modules(XConfigPtr config)
     if (config->modules == NULL)
         return FALSE;
 
-    /* make sure glx is loaded */
-
-    found = FALSE;
-    for (load = config->modules->loads; load; load = load->next) {
-        if (xconfigNameCompare("glx", load->name) == 0) found = TRUE;
-    }
-
-    if (!found) {
-        config->modules->loads =
-            xconfigAddNewLoadDirective(config->modules->loads,
-                                       "glx", XCONFIG_LOAD_MODULE,
-                                       NULL, FALSE);
-    }
+    /* make sure all our required modules are loaded */
+    ensure_module_loaded(config, "glx");
+#if defined(NV_SUNOS)
+    ensure_module_loaded(config, "xtsol");
+#endif // defined(NV_SUNOS)
 
     /* make sure GLcore and dri are not loaded */
 
@@ -82,11 +92,9 @@ int update_modules(XConfigPtr config)
     while (load) {
         next = load->next;
         if (xconfigNameCompare("GLcore", load->name) == 0) {
-            config->modules->loads =
-                xconfigRemoveLoadDirective(config->modules->loads, load);
+            xconfigRemoveLoadDirective(&config->modules->loads, load);
         } else if (xconfigNameCompare("dri", load->name) == 0) {
-            config->modules->loads =
-                xconfigRemoveLoadDirective(config->modules->loads, load);
+            xconfigRemoveLoadDirective(&config->modules->loads, load);
         }
         load = next;
     }
@@ -110,7 +118,7 @@ int update_screen(Options *op, XConfigPtr config, XConfigScreenPtr screen)
     
     update_display(op, screen);
     update_depth(op, screen);
-    update_device(config, screen->device);
+    update_device(op, config, screen->device);
     update_options(op, screen);
     
     return TRUE;
@@ -180,7 +188,8 @@ int update_extensions(Options *op, XConfigPtr config)
 
         /* remove any existing composite extension option */
         
-        remove_option_from_list(&(config->extensions->options), "Composite");
+        xconfigRemoveNamedOption(&(config->extensions->options), "Composite",
+                                 NULL);
 
         /* determine the value to set for the Composite option */
 
@@ -190,9 +199,7 @@ int update_extensions(Options *op, XConfigPtr config)
         
         /* add the option */
 
-        config->extensions->options =
-            xconfigAddNewOption(config->extensions->options,
-                                nvstrdup("Composite"), nvstrdup(value));
+        xconfigAddNewOption(&config->extensions->options, "Composite", value);
     }
     
     return TRUE;
@@ -200,6 +207,38 @@ int update_extensions(Options *op, XConfigPtr config)
 } /* update_extensions() */
 
 
+/*
+ * update_server_flags() - update the server flags section with any
+ * server flag options; the only option so far is "HandleSpecialKeys"
+ */
+
+int update_server_flags(Options *op, XConfigPtr config)
+{
+    if (!op->handle_special_keys) return TRUE;
+
+    if (!config->flags) {
+        config->flags = nvalloc(sizeof(XConfigFlagsRec));
+        if ( !config->flags ) {
+            return FALSE;
+        }
+    }
+
+    if (config->flags->options) {
+        xconfigRemoveNamedOption(&(config->flags->options),
+                                 "HandleSpecialKeys", NULL);
+    }
+
+    if (op->handle_special_keys != NV_DISABLE_STRING_OPTION) {
+        xconfigAddNewOption(&config->flags->options, "HandleSpecialKeys",
+                            op->handle_special_keys);
+    }
+
+    return TRUE;
+
+} /* update_server_flags() */
+
+
+
 
 /*
  * update_device() - update the device; there is a lot of information
@@ -210,9 +249,9 @@ int update_extensions(Options *op, XConfigPtr config)
  * the NVIDIA X driver.
  */
 
-static int update_device(XConfigPtr config, XConfigDevicePtr device)
+static int update_device(Options *op, XConfigPtr config, XConfigDevicePtr device)
 {
-    char *identifier, *vendor, *comment, *board, *busid;
+    char *identifier, *vendor, *comment, *board, *busid, *driver;
     int screen;
     XConfigDevicePtr next;
     XConfigOptionPtr options;
@@ -225,6 +264,7 @@ static int update_device(XConfigPtr config, XConfigDevicePtr device)
     screen = device->screen;
     board = device->board;
     busid = device->busid;
+    driver = device->driver;
     
     memset(device, 0, sizeof(XConfigDeviceRec));
 
@@ -240,8 +280,12 @@ static int update_device(XConfigPtr config, XConfigDevicePtr device)
     device->chipid = -1;
     device->chiprev = -1;
     device->irq = -1;
-    
-    device->driver = "nvidia";
+
+    if (op->preserve_driver) {
+        device->driver = driver;
+    } else {
+        device->driver = "nvidia";
+    }
     
     /*
      * XXX do we really want to preserve the BusID line?  Let's only
@@ -271,7 +315,8 @@ static void update_depth(Options *op, XConfigScreenPtr screen)
     
     /* update the depth */
     if ((op->depth == 8) || (op->depth == 15) ||
-        (op->depth == 16) || (op->depth == 24)) {
+        (op->depth == 16) || (op->depth == 24) ||
+        (op->depth == 30)) {
         screen->defaultdepth = op->depth;
     } else {
         /* read the default depth to SVC and set it as the default depth */
@@ -318,7 +363,7 @@ static void update_display(Options *op, XConfigScreenPtr screen)
         XConfigDisplayPtr display;
         XConfigModePtr mode = NULL;
         
-        mode = xconfigAddMode(mode, "nvidia-auto-select");
+        xconfigAddMode(&mode, "nvidia-auto-select");
         
         display = nvalloc(sizeof(XConfigDisplayRec));
         display->depth = screen->defaultdepth;
@@ -330,6 +375,7 @@ static void update_display(Options *op, XConfigScreenPtr screen)
         
         screen->displays = display;
     }
+
 } /* update_display() */
 
 
